@@ -183,7 +183,8 @@ last_login_read(pam_handle_t *pamh, int announce, int last_fd, uid_t uid)
 		ll_time = last_login.ll_time;
 		tm = localtime_r (&ll_time, &tm_buf);
 		strftime (the_time, sizeof (the_time),
-			  " %a %b %e %H:%M:%S %Z %Y", tm);
+	        /* TRANSLATORS: "strftime options for date of last login" */
+			  _(" %a %b %e %H:%M:%S %Z %Y"), tm);
 
 		date = the_time;
 	    }
@@ -274,7 +275,7 @@ last_login_write(pam_handle_t *pamh, int announce, int last_fd,
     terminal_line = get_tty(pamh);
 
     /* copy to last_login */
-    last_login.ll_host[0] = '\0';
+    last_login.ll_line[0] = '\0';
     strncat(last_login.ll_line, terminal_line, sizeof(last_login.ll_line)-1);
     terminal_line = NULL;
 
